import plotly.graph_objects as go

def add_association_markers(fig, df, subsets, scatter_x, scatter_y, max_y, params):
    fig.add_trace(go.Scatter(
        x=scatter_x,
        y=scatter_y,
        mode='markers',
        showlegend=False,
        marker=dict(
            size=params['marker_size'],
            color='#C9C9C9'
        ),
    ), row=1, col=3)
    
    for i, s in enumerate(subsets):
        scatter_x_has, scatter_y_has, marker_colors = [], [], []
        for j in range(len(df.columns)):
            if df.columns[j] in s:
                scatter_x_has.append(i)
                scatter_y_has.append(-j * max_y / len(df.columns) - 0.1 * max_y)
                marker_colors.append(params['category_color_mapping'][df.columns[j]])
                fig.add_trace(go.Scatter(
                    x=scatter_x_has,
                    y=scatter_y_has,
                    mode='markers+lines',
                    showlegend=False,
                    marker=dict(
                        size=params['marker_size'],
                        color=marker_colors,
                        line=dict(
                            color=params['markerline_color']
                        ),
                        showscale=False
                    ),
                    line=dict(
                        color=params['markerline_color'],
                        width=4
                    )
                ), row=1, col=3)
    fig.update_xaxes(showticklabels=False)
    fig.update_yaxes(showticklabels=False)
    fig.update_traces(hoverinfo=None)
    
    return fig

def add_subset_counts_bar(fig, subsets, plot_df, params):
    fig.add_trace(go.Bar(
        x=list(range(len(subsets))),
        y=plot_df['Size'],
        marker=dict(
            color=params['bar_intersect_color'],

        ),
        text=plot_df['Size'],
        textposition='outside',
        hoverinfo='none',
        textfont=dict(
            size=params['datalabel_size'],
            color=params['datalabel_color']
        )
    ), row=1, col=3
    )
    
    return fig
    
def add_category_labels(fig, df, scatter_y, params):
    max_string_len = max([len(x) for x in df.columns])
    fig.add_trace(go.Scatter(
        x=[-0.01 * max_string_len]*len(df.columns), 
        y=scatter_y,
        text=[f'<b>{col}</b>' for col in df.columns],
        mode='text',
        textposition='middle center',
        showlegend=False,
        textfont=dict(
            size=params['categorylabel_size'],
            color=params['categorylabel_color']
        )
    ), row=1, col=2
    )
    
    return fig
    
def add_category_counts_bar(fig, df, params):
    true_counts = df.sum()
    
    # create bar chart for the true counts of each overarching set 
    fig.add_trace(go.Bar(
        y=true_counts.index,
        x=-true_counts.values,
        orientation='h',
        marker_color=params['categories_colors'],
        text=true_counts.values,
        textposition='outside',
        width=0.4,
        textfont=dict(
            size=params['datalabel_size'],
            color=params['datalabel_color']
        )
    ), row=1, col=1
    )
    
    return fig, true_counts
    
def autosize(fig, true_counts):
    yaxis_domain = [0.055, 0.43] if len(true_counts.keys()) == 3 else [0.035, 0.45]
    fig.update_layout(
        plot_bgcolor='white',
        xaxis1=dict(domain=[0.1, 0.3], range=[1.75*min(-true_counts.values),0]),
        xaxis2=dict(domain=[0.32, 0.39]),
        xaxis3=dict(domain=[0.39, 0.9]),
        yaxis1=dict(domain=[0, 0.5], showticklabels=False, autorange='reversed', side='right'),
        # yaxis2=dict(domain=[0.035, 0.45]),
        # yaxis2=dict(domain=[0.055, 0.43]),
        # yaxis2=dict(domain=[0.015, 0.47]),
        yaxis2=dict(domain=yaxis_domain),
        yaxis3=dict(domain=[0, 1]),
        showlegend=False
    )
    
    return fig


DEFAULT_PARAMS = {
    'categories_colors': ['red', 'blue', 'green', 'purple', 'orange', 'yellow'],
    'categorylabel_size': 20,
    'categorylabel_color': 'black',
    'datalabel_color': 'black',
    'datalabel_size': 20,
    'marker_size': 20,
    'markerline_color': 'black',
    'markerline_size': 20,
    'bar_intersect_color': 'black',
}

def get_params(kwargs):
    
    params = {}
    
    for param in DEFAULT_PARAMS:
        params[param] = kwargs.get(param, DEFAULT_PARAMS[param])
        
    return params

def get_category_color_mapping(df, params):
    category_color_mapping = {category: color for category, color in zip(df.columns, params['categories_colors'])}
    params['category_color_mapping'] = category_color_mapping
    return params


import pandas as pd
from .upset import Upset

def demo():
    data = {
        'Class A': [True, True, True, False, False, True],
        'Class B': [True, True, True, True, True, False],
        'Class C': [False, False, False, True, True, True]
    }

    df = pd.DataFrame(data)

    upset = Upset.generate_plot(df)
    upset.show()


from plotly.subplots import make_subplots

from .set_association import *
from .plotting import * 
from .utils import *

class Upset:
    
    def generate_plot(df, **kwargs):
        
        params = get_params(kwargs)
        params = get_category_color_mapping(df, params)
        
        subsets = find_subsets(df)
        subset_sizes = find_subset_sizes(df, subsets)
        plot_df = create_plot_df(subsets, subset_sizes)
        subsets, scatter_x, scatter_y, max_y = determine_pos_x_y(df, subsets, plot_df)
        
        fig = make_subplots(1, 3, horizontal_spacing=0)
        fig = add_association_markers(fig, df, subsets, scatter_x, scatter_y, max_y, params)
        fig = add_subset_counts_bar(fig, subsets, plot_df, params)
        fig = add_category_labels(fig, df, scatter_y, params)
        fig, true_counts = add_category_counts_bar(fig, df, params)
        fig = autosize(fig, true_counts)
        
        return fig
        


from setuptools import setup, find_packages

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()


setup(
    name='upsetty',
    version='0.1.2',
    packages=find_packages(),
    install_requires=[
        'plotly',
        'pandas',
        'numpy'
    ],
    author='eskin22',
    author_email='blakepmcbride@gmail.com',
    description='An UpSet plot creation package using Plotly',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/eskin22/upsetty',
    download_url='https://pypi.org/project/upsetty/',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers', 
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
        'Topic :: Scientific/Engineering :: Visualization'
    ],
    keywords=[
        'upset',
        'upset-plot',
        'upsetplot',
        'set-interaction',
        'set-intersection',
        'set-interactions',
        'sets',
        'plotting',
        'visualization',
        'data-visualization',
        'visualization-tools',
        'data-visualization-tools',
        'python',
        'python3',
        'python310',
        'pandas',
        'plotly'
    ]
)


import itertools
import pandas as pd

def find_subsets(df):
    subsets = []
    for i in range(1, len(df.columns) + 1):
        subsets = subsets + [list(x) for x in list(itertools.combinations(df.columns, i))]
    return subsets

def find_subset_sizes(df, subsets):
    subset_sizes = []
    for s in subsets:
        curr_bool = [1] * len(df)
        for col in df.columns:
            if col in s: curr_bool = [x and y for x, y in zip(curr_bool, list(df.loc[:, col].copy()))]
            else: curr_bool = [x and not y for x, y in zip(curr_bool, list(df.loc[:, col].copy()))] 
        subset_sizes.append(sum(curr_bool))
    return subset_sizes

def create_plot_df(subsets, subset_sizes):
    plot_df = pd.DataFrame({'Intersection': subsets, 'Size': subset_sizes})
    plot_df = plot_df.sort_values(by='Size', ascending=False)
    return plot_df

def determine_pos_x_y(df, subsets, plot_df):
    max_y = max(plot_df['Size']) + 0.1 * max(plot_df['Size'])
    subsets = list(plot_df['Intersection'])
    scatter_x, scatter_y = [], []
    for i, s in enumerate(subsets):
        for j in range(len(df.columns)):
            scatter_x.append(i)
            scatter_y.append(-j * max_y / len(df.columns) - 0.1 * max_y)
    return subsets, scatter_x, scatter_y, max_y